﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Management;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Linq;
using System.Reflection;

namespace WebServices
{
    /// <summary>
    /// WMI Proxy Service
    /// 
    /// Brent Ashley, KnowledgeWorx Inc. <bashley@kwx.ca>
    ///
    /// $Id: WMIProxy.asmx.cs 7 2009-10-23 19:42:38Z brentashley $
    /// 
    /// </summary>
     
    [WebService(Namespace = "http://kwx.ca/WebServices/WMIProxy")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    // make available to ASP.NET Ajax calls:
    [System.Web.Script.Services.ScriptService]
    public class WMIProxy : System.Web.Services.WebService
    {

        /// <summary>
        /// 
        /// </summary>
        enum ErrorCodes
        {
            OK = 0,
            FAILED,
            ERROR,
            UNKNOWN
        }

        /// <summary>
        /// 
        /// </summary>
        [WebMethod(Description = "Version of the WMIProxy Service")]
        public string Version()
        {
            return ProductVersion();
        }

        /// <summary>
        /// 
        /// </summary>
        [WebMethod(Description = "System Uptime in Hours")]
        public string[] Uptime(string host)
        {
            string[] ret = new string[] { ErrorCodes.OK.ToString(), "unknown" };

            string query = "select lastbootuptime, localdatetime from Win32_OperatingSystem";

            ManagementObjectSearcher searcher = getSearcher(host, query);
            if (searcher != null)
            {
                try
                {
                    foreach (ManagementObject o in searcher.Get())
                    {
                        DateTime lastboot = ManagementDateTimeConverter.ToDateTime(o["lastbootuptime"].ToString());
                        DateTime curtime = ManagementDateTimeConverter.ToDateTime(o["localdatetime"].ToString());
                        ret[1] = Convert.ToInt32(curtime.Subtract(lastboot).TotalHours).ToString();
                    }
                }
                catch (Exception ex) 
                {

                    ret = new string[] {ErrorCodes.ERROR.ToString(), ex.Message};
                }
            }
            return ret;
        }

        /// <summary>
        /// 
        /// </summary>
        [WebMethod(Description = "Fixed Disk Percent Free Space")]
        public string[] FixedDiskPctFree(string host, string drive)
        {
            string[] ret = new string[] { ErrorCodes.OK.ToString(), "" };

            string driveletter = drive.Substring(0, 1).ToUpper();
            string query = 
                "SELECT freespace, size FROM Win32_LogicalDisk " +  
                "where mediatype=12 and " +
                "deviceid='" + driveletter + ":'";
            try
            {
                ManagementObjectSearcher searcher = getSearcher(host, query);
                if (searcher != null)
                {
                    foreach (ManagementObject o in searcher.Get())
                    {
                        Int64 freespace = Convert.ToInt64(o["freespace"]);
                        Int64 size = Convert.ToInt64(o["size"]);
                        Int64 pct = (100 * freespace) / size;
                        ret[1] = pct.ToString();
                    }
                    if (ret[1] == "")
                    {
                        ret = new string[] { ErrorCodes.ERROR.ToString(), "Fixed Disk " + driveletter + ": not found" };
                    }
                }
                else
                {
                    ret = new string[] { ErrorCodes.FAILED.ToString(), "No results returned" };
                }
            }
            catch (Exception ex)
            {
                ret = new string[] { ErrorCodes.ERROR.ToString(), ex.Message };
            }
            return ret;        
            
        }

        /// <summary>
        /// 
        /// </summary>
        [WebMethod(Description = "Ping from host to destination")]
        public string[] PingHostToDestination(string host, string destination)
        {
            string[] ret = new string[] { ErrorCodes.OK.ToString(), "" };

            string query = "select StatusCode,ResponseTime from Win32_PingStatus where address = '" + destination + "'";
            try
            {
                ManagementObjectSearcher searcher = getSearcher(host, query);
                if (searcher != null)
                {
                    foreach (ManagementObject o in searcher.Get())
                    {
                        Int64 statusCode = Convert.ToInt64(o["StatusCode"]);
                        if (statusCode == 0)
                        {
                            ret[1] = Convert.ToInt64(o["ResponseTime"]).ToString();
                        } else
                            ret = new string[] { ErrorCodes.ERROR.ToString(), statusCode.ToString() };

                    }
                }
                else
                {
                    ret = new string[] { ErrorCodes.FAILED.ToString(), "No results returned" };
                }
            }
            catch (Exception ex)
            {
                ret = new string[] { ErrorCodes.ERROR.ToString(), ex.Message };
            }
            return ret;
        }

        /// <summary>
        /// 
        /// </summary>
        [WebMethod(Description = "This method checks free amount of physical memory and pagefile, returns FreePhysicalMemory:#,FreeVirtualMemory:#,TotalVirtualMemorySize:#,TotalPhysicalMemory:#")]
        public string[] Memory(string host)
        {
            string[] ret = new string[] { ErrorCodes.OK.ToString(), "" };
            string query = "Select FreePhysicalMemory,FreeVirtualMemory,TotalVirtualMemorySize from Win32_OperatingSystem";

            Int64 availablePMem = 0;
            Int64 availableVMem = 0;
            Int64 totalVMem = 0;
            Int64 totalPMem = 0;

            ManagementObjectSearcher searcher = getSearcher(host, query);
            try
            {
                if (searcher != null)
                {
                    foreach (ManagementObject o in searcher.Get())
                    {
                        availablePMem = Convert.ToInt64(o["FreePhysicalMemory"].ToString()) * 1024;
                        availableVMem = Convert.ToInt64(o["FreeVirtualMemory"].ToString()) * 1024;
                        totalVMem = Convert.ToInt64(o["TotalVirtualMemorySize"].ToString()) * 1024;
                    }
                }

                query = "Select * from Win32_ComputerSystem";
                searcher = getSearcher(host, query);

                if (searcher != null)
                {
                    foreach (ManagementObject o in searcher.Get())
                    {
                        totalPMem += Convert.ToInt64(o["TotalPhysicalMemory"].ToString()) * 1024;
                    }
                }

                ret[1] = string.Format("FreePhysicalMemory:{0},FreeVirtualMemory:{1},TotalVirtualMemorySize:{2},TotalPhysicalMemory:{3}",
                    availablePMem,availableVMem,totalVMem,totalPMem);
            
            }
            catch (Exception ex)
            {
                ret = new string[] { ErrorCodes.FAILED.ToString(), ex.Message };
            }

            return ret;
        }

        /// <summary>
        /// 
        /// </summary>
        [WebMethod(Description = "This method checks the number of messages in Exchange Queues")]
        public string[] ExchangeQueues(string host)
        {
            string[] ret = new string[] { ErrorCodes.OK.ToString(), "" };

            string query = "Select * from Exchange_SMTPQueue where QueueName = 'LocalAsyncQueue'";
            try
            {
                ManagementObjectSearcher searcher = getSearcher(host, query,"\\root\\MicrosoftExchangeV2:Exchange_SMTPQueue");
                if (searcher != null)
                {
                    bool found = false;
                    foreach (ManagementObject o in searcher.Get())
                    {
                        found = true;
                        string queueName = o["LinkName"].ToString();
                        string queueSize = o["MessageCount"].ToString();
                        ret[1] = string.Format("'{0}' has {1} messages\n", queueName, queueSize);
                    }
                    if (!found)
                    {
                        ret = new string[] {ErrorCodes.ERROR.ToString(), "Queue not found"};
                    }
                }
            }
            catch (Exception ex)
            {
                ret = new string[] { ErrorCodes.ERROR.ToString(), ex.Message };
            }


            return ret;

        }


        /// <summary>
        /// 
        /// </summary>
        [WebMethod(Description = "Gets the state of a service, returns Running:Stopped:Paused")]
        public string[] ServiceCheck(string host, string servicename)
        {
            string[] ret = new string[] { ErrorCodes.OK.ToString(), "" };

            string query = "SELECT * FROM Win32_Service WHERE Name = '" + servicename.Trim() + "'";
            try
            {
                ManagementObjectSearcher searcher = getSearcher(host, query);
                if (searcher != null)
                {
                    bool found = false;
                    foreach (ManagementObject o in searcher.Get())
                    {
                        found = true;
                        string state = (string)o["State"];
                        ret[1] += state;
                    }
                    if (!found)
                        ret = new string[] { ErrorCodes.UNKNOWN.ToString(), "Service Not Found" };
                }
            }
            catch (Exception ex)
            {
                ret = new string[] { ErrorCodes.ERROR.ToString(), ex.Message };
            }


            return ret;

        }





        /// <summary>
        /// 
        /// </summary>
        [WebMethod(Description = "Execute WMI Query")]
        public string[] Query(string host, string query)
        {
            string[] ret = new string[] { ErrorCodes.OK.ToString(), "" };
            try
            {
                ManagementObjectSearcher searcher = getSearcher(host, query);
                if (searcher != null)
                {
                    foreach (ManagementObject o in searcher.Get())
                    {
                        ret[1] += o.GetText(TextFormat.Mof) + "\n";
                    }
                }
            }
            catch (Exception ex)
            {
                ret = new string[] { ErrorCodes.ERROR.ToString(), ex.Message };
            }
            return ret;
        }

        /// <summary>
        /// 
        /// </summary>
        protected ManagementObjectSearcher getSearcher(string host, string query)
        {
            return getSearcher(host, query, "\\root\\CIMV2");
        }

        /// <summary>
        /// 
        /// </summary>
        protected ManagementObjectSearcher getSearcher(string host, string query, string _namespace)
        {
            ObjectQuery objquery = new ObjectQuery(query);

            EnumerationOptions enumopts = new EnumerationOptions();
            enumopts.ReturnImmediately = true;
            enumopts.Rewindable = false;
            enumopts.Timeout = new TimeSpan(0, 0, Convert.ToInt16(ConfigurationManager.AppSettings["timeout"]));

            ManagementScope scope = getScope(host,_namespace);
            return new ManagementObjectSearcher(scope, objquery, enumopts);
        }

        /// <summary>
        /// 
        /// </summary>
        protected ManagementScope getScope(string host, string _namespace)
        {
            ManagementPath path = new ManagementPath(@"\\" + host + _namespace);
            return new ManagementScope(path, getOptions(host));
        }

        /// <summary>
        /// 
        /// </summary>
        protected ConnectionOptions getOptions(string host)
        {
            // host is passed so we can add per-host credential lookup later
            ConnectionOptions options = new ConnectionOptions();

            string creds = "";
            try
            {
                creds = ConfigurationManager.AppSettings["creds for " + host];
            } 
            catch {}

            if (String.IsNullOrEmpty(creds))
            {
                creds = ConfigurationManager.AppSettings["default creds"];
            }
            
            string[] credArray = creds.Split(":".ToCharArray());
            options.Username = credArray[0];
            options.Password = credArray[1];

            return options;
        }

        /// <summary>
        /// 
        /// </summary>
        protected static string ProductVersion()
        {
            return Assembly.GetExecutingAssembly().GetName().Version.ToString();
        }
    }
}
